/**
 * Practice: some common programming idioms based
 * upon manipulating arrays ...
 */

var newArray=Array(); // global variable that is shared.
var tempArray1, temparray2; // arrays that hold "intermediate"
 // values --for testing swap, etc.
/* A note on "global" variables.
 * Considered a bad practice in most modern programming texts,
 * global variables expose implementation details to anyone
 * who has access to the code. This means that any function
 * can change the value of the variable from any place within
 * the code, which results in difficult to understand/maintain
 * software.
 *
 * Because the DOM/JavaScript interaction model is as it is,
 * however, you will see frequent use of such constructions
 * because ... well, it makes interprocess/interfunction
 * communication easier.
 */

//__________Methods__________

/*
 * Note: each method is paired with its "event handler,"
 * which is the method that is called from the Document,
 * i.e., the HTML.
 */
// ___private, helper methods, for input/output.
function formatArray( anyArray ) {
   if( ! validateInitalArray() ) {
      return "The Empty Array";
   }
   var description="[ ";
   for( var index=0; index < anyArray.length - 1; index++ ) {
      description += anyArray[ index ] + ", ";
   }
   return description + anyArray[ anyArray.length -1 ] + " ]";
}

function formatArrayUpToIndex( anArray, toIndex ) {
   if( ! validateInitalArray() ) {
      return "The Empty Array";
   }
   var start=0;
   if( toIndex < 0 ) {
      return "Target was not found in " + formatArray( anArray );
   }
   var returnString="[";
   for( ; start < toIndex; start++ ) {
      returnString += anArray[ start ] +", ";
   }
   if( start < anArray.length ) {
      return returnString + anArray[ start ] + " ... ]";
   } else {
      return returnString + " ... ]";
   }
}

/**
 * In the event that the users does not go through the genArray
 * dialogue, they will get an Array() object, which is the
 * default ctor's idea of an empty array, I suppose.
 * It looks like [ undefined ]. So, I will create a
 * predicate that looks for any array of length 0
 * (assuming that is what the property will return) and
 * use that predicate to provide more meaningful feedback
 * to the user.
 */
function mtArray( array ) {
   return Array.isArray( array ) && array.length===0;
}
/**
 * called only by printing routines that
 * wish to provide additional information.
*/
function validateInitalArray() {
   if( mtArray( newArray ) ) {
      window.alert("It appears that you have NOT defined (generated) a new Array for this session. All methods will likely return undefined results.");
      return false;
   }
   return true;
}
//___private, helper methods ... end here.

/** ==> User provided methods paired with event-handers <=== **/
/**
 * preconditions: n is a non-negative integer.
 * postconditions: a new array of size n is created and
 * populated with randomly generated integers, starting from 0
 * through n-1.
 *
 * A special note on using the Math.random() function:
 *    Math.random() returns a double (decimal) number between 0.0 and 1.0.
 *    Typically, we wish to generate random integers in a particular
 *    range: such as 0 through N. One way to do this is to
 *    multiply the Math.random() * N, which gives us another decimal
 *    number. You will then need to discard the decimal component;
 *    the Math.floor() function does just that.
 */
function genArray( n ) {
   window.alert("You must implement this function.");
   return newArray;
}
/*
 * Event Handler for genArray function:
 * Called from the document; feeds the genArray function
 * preconditions: input is non-negative integer, perhaps
 * no larger than 100 for sanity's sake ...
 * Note: revision for version 2: added some mtArray checking
 * and recovery scripts, see notes under mtArray and friends,
 * above.
 */
function testGenArray() {
   var size = parseInt( document.getElementById("input").value );
   var generatedArray = genArray( size );
   newArray = generatedArray; // remember this array for remainder of the exercises.
   document.getElementById("exercise-1").innerHTML=formatArray( generatedArray );
}
/**
 * preconditions: the top-level newArray variable has been set to
 * a valid array as a result of completing the previous
 * exercise.
 * postconditions: the "index" of the "target" is returned; if the
 * target is not found in the newArray, then -1 is returned.
 */
function indexOf( target ) {
    window.alert("You must implement this function.");
   return -1;
}
/**
 * Event Handler for indexOf function.
 * Retrieves input, tests your indexOf function, and
 * pretty prints the results into the appropriate field.
 */
function testIndexOf() {
   var searchForTarget = parseInt( document.getElementById("target").value);
   var locationInArray = indexOf( searchForTarget );
   document.getElementById("exercise-2").innerHTML = formatArrayUpToIndex( newArray, locationInArray );
}

/**
 * preconditions: the top-level newArray must have been created and must
 * contain integers.
 * postconditions: a new array (possibly empty) that contains only the even integers
 * that appeared in the top-level newArray is returned. Note: this function does
 * not modify the top-level newArray!!
 */
function filterEvens() {
    window.alert("You must implement this function.");
}

/**
 * Event Handler: onlick.
 * Tests the filterEvens() function.
 */
function testFilterEvens() {
   var onlyEvens = filterEvens();
   document.getElementById("exercise-3").innerHTML = formatArray( onlyEvens );
}
/**
 * preconditions: anArray should be non-empty, and the parameters to and from must
 * be valid indexes for anArray (less than anArray.length).
 */
function swap( to, from, anArray ) {
    window.alert("You must implement this function.");
}
// misc. function used to ensure separation
// of arrays (non-interference between arrays
// for swap testing.)
// DO NOT modify this function.
function copyArray( theArray ) {
   var copy = Array( theArray.length );
   for( var index = 0; index < theArray.length; index++ ) {
      copy[ index ] = theArray[ index ];
   }
   return copy;
}
/**
 * Event Handler: onlick.
 * Tests the swap() procedure.
 */
function testSwap() {
   tempArray1 = genArray( 4 );
   tempArray2 = copyArray( tempArray1 );
   swap( 1, 3, tempArray1 );
   document.getElementById("exercise-4").innerHTML="Original array: " + formatArray( tempArray2 ) + " swapped elements at index 1 and index 3 => " + formatArray( tempArray1 );
}

/*
 * additional methods required to implement the selection/insertion sort
 * routines.
 */

/*
 * Event Handler: do not modify this function.
 */
function doSelectSort() {
   /* shallow copies the gloval array so that it remains
    * unaffected by the call to the sorting routine.
    */
   var sortedArray = copyArray( newArray );
   selectionSort( sortedArray );
   document.getElementById("selectSortResults").style.visibility="visible";
   document.getElementById("selectSortResults").innerHTML=formatArray( sortedArray );
}
/*
 * Main routine: you write this:
 * precondition(s):
 * postcondition: Sorts the array in ascending order using the selection sort algorithm.
 */
function selectionSort( array ) {
    window.alert("You must implement this function.");
}
/*
 * Helper function: you write this.
 * precondition(s):
 * postconditions: returns the index of the smallest element in the
 * array, searching from the "start" index to the last element of the array.
 */
function findMinIndex( start, array ) {
    window.alert("You must implement this function.");
}

/*
 *Sorting by selection: you need to write this function.
 *preconditions: an array of integers (possibly empty) is provided.
 *postcondition: the array is sorted in ascending order, in place (in
 *other words, the original array passed to this function is modified
 *by this function.)
 */
function insertionSort( array ) {
   window.alert("You must implement this function.");
}

/*
 * Event Handler: doInsertionSort.
 * (do not modify this code)
 * Handles the button interaction by copying the newArray and arranging to have
 * it sent to the insertionSort function (above) and the results displayed
 * in the expected place.
 */
function doInsertSort() {
   var safeArray = copyArray( newArray );
   insertionSort( safeArray );
   document.getElementById("insertSortResults").style.visibility="visible";
   document.getElementById("insertSortResults").innerHTML=formatArray( safeArray );
}
